import { DeleteOptions, GetOptions, ListOptions, PagedItems, SubscriptionResponse, UserInfoResponse } from './common';
import { Configuration } from './config';
import { Policy } from './policy';
import { ServiceResponse } from './response';
import { Service } from './service';
export declare const DefaultOrganizerServiceUri = "https://org-api.connect.trimble.com/v1/";
/**
 * The request body for the @see Organiser#createTree operation.
 */
export interface CreateTreeRequest {
    /** The optional identifier for the new tree, if not provided service will generate an unique identifier. */
    id?: string;
    /** The tree short name. */
    name: string;
    /** The optional description. */
    description?: string;
    /** The type of the tree that defines the symantic. */
    type: string;
}
/**
 * The request parameters for the @see Organiser#getTree operation.
 */
export interface GetTreeOptions extends GetOptions {
}
/**
 * The request parameters for the @see Organiser#getNode operation.
 */
export interface GetNodeOptions extends GetOptions {
}
/**
 * The request body for the @see Organiser#updateTree operation.
 * All properties are optional, if property is not provided in the request the property will retain it's old value.
 */
export interface UpdateTreeRequest {
    /**
     * The last seen version of the tree descriptor.
     * If provided service will use this information for the concurency control and will refuse to apply changes if current version is not equal to version provided in the request.
     */
    v?: number;
    /** The new name. */
    name?: string;
    /** The new description. */
    description?: string;
    /** The new type. */
    type?: string;
}
/**
 * The tree descriptor returned from @see Organiser#createTree , @see Organiser#updateTree , @see Organiser#deleteTree and @see Organiser#getTree operations.
 */
export interface TreeResponse {
    /** The unique tree identifier. */
    id: string;
    /** The creation timestamp in ISO format. */
    createdAt: string;
    /** The identifier of the principal who created the tree. */
    createdBy: string;
    /** The last modification timestamp in ISO format. */
    modifiedAt: string;
    /** The identifier of the princilat who last modified the tree descriptor. */
    modifiedBy: string;
    /** The current version of the tree descriptor. */
    v: number;
    /** The deletion marker. If not present or false the tree is active, otherwise the tree is marked as deleted. */
    deleted?: boolean;
    /** The tree short name. */
    name: string;
    /** The optional description. */
    description?: string;
    /** The tree type that defines the symantic. */
    type: string;
}
/**
 * The request body for the @see Organiser#createNode operation.
 */
export interface CreateNodeRequest {
    /** The optional identifier for the new tree, if not provided service will generate an unique identifier. */
    id?: string;
    /** The node name. To pesent a tree-like structure this name is interpreted as a full path in a tree with '/' character as a separator between path segments. */
    name: string;
    /** The optional collection of links to resources to associate with this node. */
    links?: string[];
}
/**
 * The request body for the @see Organiser#updateNode operation.
 * All properties are optional, if property not provided in the request the property will retain it's old value.
 */
export interface UpdateNodeRequest {
    /**
     * The last seen version of the tree descriptor.
     * If provided service will use this information for the concurency control and will refuse to apply changes if current version is not equal to version provided in the request.
     */
    v?: number;
    /** The new name. */
    name?: string;
    /** The new set of links to resources. If this property is provided it will override all existing links associated. For differential changes use @see add and @see remove properties. */
    links?: string[];
    /** The set of links to add to the associations collection (@see NodeResponse#links ) all currently associated links remain unchanged. */
    add?: string[];
    /** The set of links to be removef from to the associations collection (@see NodeResponse#links ). Links that are not associated already, are ignored. */
    remove?: string[];
}
/**
 * The tree descriptor returned from @see Organiser#createNode, @see Organiser#updateNode, @see Organiser#deleteNode and @see Organiser#getNode operations.
 */
export interface NodeResponse {
    /** The unique node identifier. */
    id: string;
    /** The creation timestamp in ISO format. */
    createdAt: string;
    /** The identifier of the principal who created the node. */
    createdBy: string;
    /** The last modification timestamp in ISO format. */
    modifiedAt: string;
    /** The identifier of the princilat who last modified the node. */
    modifiedBy: string;
    /** The version of the node. */
    v: number;
    /** The deletion marker. If not present or false the node is active, otherwise the node is marked as deleted. */
    deleted?: boolean;
    /** The name. */
    name: string;
    /**
     * The collection of links to resources to associate with this node. Might be absent if client requested to skip this property from the response.
     * If property is requested and there are no associations then empty array is returned.
     */
    links?: string[];
    /** The number of links associated with the node. */
    count: number;
}
/**
 * The de-serialized data returned from the request with list of tree descriptors.
 */
export interface ListTreesResponse extends PagedItems<TreeResponse> {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListNodesResponse extends PagedItems<NodeResponse> {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListTreesOptions extends ListOptions {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListNodesOptions extends ListOptions {
    /** The entity to serach for in the links collection. If specified only nodes that are linking to that entity are returned. */
    link?: string;
    /** The node name prefix to filter nodes. Only nodes whos name starts with this prefix are returned. Allows conviniently ferch branches of the tree. */
    prefix?: string;
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The de-serialized data returned from the request with list of node versions.
 */
export interface ListNodeVersionsOptions extends ListOptions {
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The changeset request parameters .
 */
export interface ChangesetOptions {
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The change node request is an upsert type of request.
 * It could be that it creates a new node or updates existing or even deletes.
 */
export declare type ChangeNodeRequest = CreateNodeRequest | UpdateNodeRequest | {
    id: string;
    deleted?: boolean;
};
/**
 * The changeset request.
 * The changeset can be applied in snapshot or incremental mode.
 * In the snapshot mode all nodes not mentioned int he requests will be marked as deleted.
 * In the incremental mode nodes not mentioned in the request are not touched.
 */
export interface ChangesetRequest {
    /** The changeset mode. If not specified the snaphsot mode is used. */
    incremental?: boolean;
    /** The node name prefix that will be used as a scope fot th changeset. Used only in the snapshot mode to limit the nodes that will be implicitly deleted. */
    scopePrefix?: string;
    /** The collection of nodes to add, update or delete. */
    nodes: ChangeNodeRequest[];
}
/**
 * The changeset response.
 */
export interface ChangesetResponse {
    /** The changeset mode was used. */
    incremental: boolean;
    /** The node name prefix was used. */
    scopePrefix?: string;
    /** The collection of nodes that were left unprocessed because of errors. */
    unprocessed: UnprocessedNode[];
    /** The collection of nodes changed as a result of the operation. */
    nodes: NodeResponse[];
}
/**
 * The node unprocessed due to some error.
 */
export interface UnprocessedNode {
    /** The node change requested in the changeset explicitly or implicitly (e.g. node not mentioned in the snaphot type of changeset request). */
    item: ChangeNodeRequest;
    /** The error code. */
    code: string;
    /** The error message. */
    message: string;
}
/**
 * The service client that represents connection to the Organizer Service.
 * Each API operation is exposed as a function on service.
 */
export declare class Organizer extends Service {
    /**
     * @constructor Constructs a service object.
     * @param {Config} config The configuration options (e.g. service url). If not provided the default configuration will be used.
     * @param {Credentials} credentials The credentials to be attached to service requests.
     */
    constructor(config?: Partial<Configuration>);
    /**
     * Returns information the service knows about the current user.
     * @returns {Promise<ServiceResponse<UserInfoResponse>>} the information known about the caller.
     */
    getUserInfo(): Promise<ServiceResponse<UserInfoResponse>>;
    /**
     * Returns some or all (up to 500, default is 100) of the trees in a forest.
     * You can use the request parameters as selection criteria to return a subset of the trees in a forest.
     * @param {string} forestId The identifier of the forest to list trees from.
     * @param {ListTreesOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListTreesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listTrees(forestId: string, params?: ListTreesOptions): Promise<ServiceResponse<ListTreesResponse>>;
    /**
     * Returns the tree descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {GetTreeOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    getTree(forestId: string, treeId: string, params?: GetTreeOptions): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Creates a new tree.
     * @param {string} forestId The identifier of the forest to create a tree in.
     * @param {CreateTreeRequest} request The tree creation request.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    createTree(forestId: string, request: CreateTreeRequest): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Updates the tree descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {UpdateTreeRequest} request The tree descriptor update request.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    updateTree(forestId: string, treeId: string, request: UpdateTreeRequest): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Marks the tree as deleted.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @returns {Promise<TreeResponse>>} The de-serialized service response with tree descriptor.
     * @param {DeleteOptions} params The optional request parameters.
     * @throws {ServiceError} in case of the error response from the service.
     */
    deleteTree(forestId: string, treeId: string, params?: DeleteOptions): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Get the tree access control policy.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @returns {Promise<ServiceResponse<Policy>>} the access control policy.
     */
    getTreePolicy(forestId: string, treeId: string): Promise<ServiceResponse<Policy>>;
    /**
     * Puts the tree access control policy.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {Policy} policy The policy.
     * @returns {Promise<ServiceResponse<void>>} the empty response.
     */
    putTreePolicy(forestId: string, treeId: string, policy: Policy): Promise<ServiceResponse<void>>;
    /**
     * Returns some or all (up to 500, default is 100) of the nodes in a tree.
     * You can use the request parameters as selection criteria to return a subset of the nodes in a tree.
     * @param {string} forestId The identifier of the forest.
     * @param {string} treeId The identifier of the tree to list trees from.
     * @param {ListNodesOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListNodesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listNodes(forestId: string, treeId: string, params?: ListNodesOptions): Promise<ServiceResponse<ListNodesResponse>>;
    /**
     * Returns some or all (up to 500, default is 100) of the node versions.
     * @param {string} forestId The identifier of the forest.
     * @param {string} treeId The identifier of the tree to list trees from.
     * @param {string} nodeId The node identifier.
     * @param {ListNodeVersionsOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListNodesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listNodeVersions(forestId: string, treeId: string, nodeId: string, params?: ListNodeVersionsOptions): Promise<ServiceResponse<ListNodesResponse>>;
    /**
     * Returns the tree descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {GetNodeOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    getNode(forestId: string, treeId: string, nodeId: string, params?: GetNodeOptions): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Creates a new node.
     * @param {string} forestId The identifier of the forest to create a tree in.
     * @param {string} treeId The identifier of the forest to create a tree in.
     * @param {CreateNodeRequest} request The tree creation request.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    createNode(forestId: string, treeId: string, node: CreateNodeRequest): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Updates the node.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {UpdateNodeRequest} request The tree descriptor update request.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    updateNode(forestId: string, treeId: string, nodeId: string, node: UpdateNodeRequest): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Marks the node as deleted.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {DeleteOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response with node descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    deleteNode(forestId: string, treeId: string, nodeId: string, params?: DeleteOptions): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Applies a changeset to a tree.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {ChangesetRequest} changeset The request.
     * @returns {Promise<ServiceResponse<ChangesetResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    changeset(forestId: string, treeId: string, changeset: ChangesetRequest, params?: ChangesetOptions): Promise<ServiceResponse<ChangesetResponse>>;
    /**
     * Requests credential to subscribe to change notifications.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The optional tree identifier.
     * @param {string} nodeId The optional node identifier.
     * @returns {Promise<ServiceResponse<SubscriptionResponse>>} The de-serialized service response with temporary credentials and subscription details.
     * @throws {ServiceError} in case of the error response from the service.
     */
    subscribe(forestId: string, treeId?: string, nodeId?: string): Promise<ServiceResponse<SubscriptionResponse>>;
}
/**
 * The default instance of the Organizer Service client.
 */
export declare const OrganizerClient: Organizer;
export default OrganizerClient;
